@@ -8,13 +8,11 @@ def clone_instance(instance, fieldnames=None):
 
         .. warning: All fields are copied, even primary key
 
-    :param instance: `django.db.models.Model`_ instance
-    :return: `django.db.models.Model`_ instance
+    :param instance: :py:class:`django.db.models.Model` instance
+    :return: :py:class:`django.db.models.Model` instance
     """
 
     if fieldnames is None:
-        # fields = [fld for fld in instance._meta.fields if fld.name in fieldnames]
-    # else:
         fieldnames = [fld.name for fld in instance._meta.fields]
 
     new_kwargs = dict([(name, getattr(instance, name)) for name in fieldnames])
@@ -25,7 +23,44 @@ def get_copy_of_instance(instance):
     return instance.__class__.objects.get(pk=instance.pk)
 
 
-def get_field_value(obj, field, usedisplay=True):
+def get_attr(obj, attr, default=None):
+    """Recursive get object's attribute. May use dot notation.
+
+    >>> class C(object): pass
+    >>> a = C()
+    >>> a.b = C()
+    >>> a.b.c = 4
+    >>> get_attr(a, 'b.c')
+    4
+
+    >>> get_attr(a, 'b.c.y', None)
+
+    >>> get_attr(a, 'b.c.y', 1)
+    1
+    """
+    if '.' not in attr:
+        ret = getattr(obj, attr, default)
+    else:
+        L = attr.split('.')
+        ret = get_attr(getattr(obj, L[0], default), '.'.join(L[1:]), default)
+
+    if isinstance(ret, BaseException):
+        raise ret
+    return ret
+
+
+def getattr_or_item(obj, name):
+    try:
+        ret = get_attr(obj, name, AttributeError())
+    except AttributeError:
+        try:
+            ret = obj[name]
+        except KeyError:
+            raise AttributeError("%s object has no attribute/item '%s'" % (obj.__class__.__name__, name))
+    return ret
+
+
+def get_field_value(obj, field, usedisplay=True, raw_callable=False):
     """
     returns the field value or field representation if get_FIELD_display exists
 
@@ -47,10 +82,13 @@ def get_field_value(obj, field, usedisplay=True):
     else:
         raise ValueError('Invalid value for parameter `field`: Should be a field name or a Field instance ')
 
-    if hasattr(obj, 'get_%s_display' % fieldname) and usedisplay:
+    if usedisplay and hasattr(obj, 'get_%s_display' % fieldname):
         value = getattr(obj, 'get_%s_display' % fieldname)()
     else:
-        value = getattr(obj, fieldname)
+        value = getattr_or_item(obj, fieldname)
+
+    if not raw_callable and callable(value):
+        return value()
 
     return value
 
@@ -117,8 +155,16 @@ def get_verbose_name(model_or_queryset, field):
     username
     >>> print unicode(get_verbose_name(User.objects.all(), 'username'))
     username
+    >>> print unicode(get_verbose_name(User.objects, 'username'))
+    username
+    >>> print unicode(get_verbose_name(User.objects, user._meta.get_field_by_name('username')[0]))
+    username
     >>> print unicode(get_verbose_name(p, 'content_type.model'))
     python model class name
+    >>> get_verbose_name(object, 'aaa')
+    Traceback (most recent call last):
+    ...
+    ValueError: `get_verbose_name` expects Manager, Queryset or Model as first parameter (got <type 'type'>)
     """
 
     if isinstance(model_or_queryset, models.Manager):
@@ -138,7 +184,7 @@ def get_verbose_name(model_or_queryset, field):
     elif isinstance(field, models.Field):
         field = field
     else:
-        raise ValueError('`get_verbose_name` field_path muset be string or Field class')
+        raise ValueError('`get_verbose_name` field_path must be string or Field class')
 
     return field.verbose_name
 
